//
// Copyright (c) 2022, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

using MindFusion.Diagramming.Layout;


namespace MindFusion.Diagramming.WinForms.Samples.CS.DecisionLayout
{
    public partial class MainForm : Form
    {
		public MainForm()
		{
			InitializeComponent();

			diagram1.EffectsProvider = new EffectsProvider();

			diagramView1.AllowInplaceEdit = true;

            terminalBrush = new MindFusion.Drawing.LinearGradientBrush(
                Color.FromArgb(224, 233, 233), Color.FromArgb(206, 0, 0), 90);
            decisionBrush = new MindFusion.Drawing.LinearGradientBrush(
                Color.FromArgb(224, 233, 233), Color.FromArgb(97, 106, 127), 90);
            normalBrush = new MindFusion.Drawing.LinearGradientBrush(
                Color.FromArgb(224, 233, 233), Color.FromArgb(156, 170, 198), 90);
            terminalPen = new MindFusion.Drawing.Pen(Color.FromArgb(206, 0, 0));
			terminalPen.Width = 0.6f;
			terminalPen.CompoundArray = new float[] { 0, 0.2f, 0.8f, 1 };

			diagram1.DiagramStyle.ShadowBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(0x6E, Color.Black));
			diagram1.DiagramStyle.FontFamily = "Arial";
			diagram1.DiagramStyle.FontSize = 7;
			diagram1.ShadowOffsetX = 2;
			diagram1.ShadowOffsetY = 2;
            diagram1.DiagramLinkStyle.Brush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(90, 121, 165));
            diagram1.DiagramLinkStyle.Stroke = new MindFusion.Drawing.SolidBrush(Color.FromArgb(45, 57, 86));
			diagram1.DiagramLinkStyle.ShadowBrush = new MindFusion.Drawing.SolidBrush(Color.Transparent);
			diagram1.DiagramLinkStyle.TextBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(0x00, 0x00, 0x8B));
			diagram1.DiagramLinkStyle.FontStyle = FontStyle.Bold;
			diagram1.LinkHeadShape = ArrowHeads.Triangle;
			diagram1.LinkHeadShapeSize = 3.5f;
			diagram1.ShapeNodeStyle.Effects.Add(new GlassEffect());
			diagram1.ShapeNodeStyle.Effects.Add(new SmoothShadowEffect());
		}

		private void layoutButton_Click(object sender, EventArgs e)
		{
			Arrange();
		}

		private void Arrange()
		{
			var layout = new MindFusion.Diagramming.Layout.DecisionLayout();
			layout.HorizontalPadding = 10;
			layout.VerticalPadding = 10;
			layout.StartNode = diagram1.FindNode("start");
			layout.Arrange(diagram1);
		}

		private void loadSampleButton_Click(object sender, EventArgs e)
		{
			var diagramFile = System.IO.Path.Combine(System.IO.Path.GetDirectoryName(Application.ExecutablePath), "flowchart.xml");
			if (System.IO.File.Exists(diagramFile))
				diagram1.LoadFromXml(diagramFile);
			else
				MessageBox.Show("Cannot find '" + diagramFile + "'.");
		}

		private void createRandomButton_Click(object sender, EventArgs e)
		{
			diagram1.ClearAll();

			var startNode = diagram1.Factory.CreateShapeNode(0, 0, 30, 10);
			startNode.Brush = terminalBrush;
			startNode.Pen = terminalPen;
			startNode.Text = "START";
			startNode.Tag = "start";
			startNode.Shape = Shapes.Ellipse;

			var endNode = diagram1.Factory.CreateShapeNode(0, 0, 30, 10);
			endNode.Brush = terminalBrush;
			endNode.Pen = terminalPen;
			endNode.Text = "END";
			endNode.Shape = Shapes.Ellipse;

			const int maxLength = 10;
			var all = new List<ShapeNode>();
			var remaining = new Queue<List<ShapeNode>>();

			remaining.Enqueue(new List<ShapeNode>(new ShapeNode[] { startNode }));

			var random = new Random(DateTime.Now.Millisecond);
			int length = 0;
			while (remaining.Count > 0)
			{
				var newNodes = new List<ShapeNode>();
				bool createdNew = false;

				var next = remaining.Dequeue();
				for (var n = 0; n < next.Count; n++)
				{
					var node = next[n];
					var isDecision = node.Tag is bool;

					ShapeNode previousChoice = null;
					int outLinks = isDecision ? 2 : 1;
					string[] linkTexts = isDecision ? new string[] { "YES", "NO" } : new string[] { "" };
					for (int i = 0; i < outLinks; i++)
					{
						// Create new or link to an existing node;
						// Create at least one new node for each level
						bool createNew = random.Next(maxLength) > length;
						if (!createdNew && n == next.Count - 1)
							createNew = true;
						if (length == maxLength)
							createNew = false;

						DiagramLink link;
						if (createNew)
						{
							// New decision or an operation node
							bool newDecision = random.Next(3) > 0 && !isDecision;
							var newNode = diagram1.Factory.CreateShapeNode(0, 0, 30, 10);
							if (newDecision)
							{
								newNode.Shape = Shapes.Decision;
								newNode.AnchorPattern = AnchorPattern.Decision2In2Out;
								newNode.Brush = decisionBrush;
								newNode.Text = "Decision";
								newNode.Tag = true;
							}
							else
							{
								newNode.Shape = Shapes.Rectangle;
								newNode.Brush = normalBrush;
								newNode.Text = "Operation";
							}

							link = diagram1.Factory.CreateDiagramLink(node, newNode);
							newNodes.Add(newNode);
							createdNew = true;
						}
						else
						{
							// Link to an existing node. If length == maxLength, link to the end node
							if (length == maxLength || remaining.Count == 0)
							{
								link = diagram1.Factory.CreateDiagramLink(node, endNode);
							}
							else
							{
								// Make sure both choices don't lead to the same node
								ShapeNode choice = null;
								while (choice == previousChoice)
									choice = all[random.Next(all.Count)];
								link = diagram1.Factory.CreateDiagramLink(node, choice);
								previousChoice = choice;
							}
						}

						link.Text = linkTexts[i];
					}

					all.Add(node);
				}

				if (newNodes.Count > 0)
					remaining.Enqueue(newNodes);

				length++;
			}

			Arrange();
		}


		private MindFusion.Drawing.Brush terminalBrush;
		private MindFusion.Drawing.Brush decisionBrush;
		private MindFusion.Drawing.Brush normalBrush;
		private MindFusion.Drawing.Pen terminalPen;
   }
}